function cv = dunnett_unequal(K,N,alpha)
    % Code to accompany "Statistical Decision Properties of Imprecise 
    % Trials Assessing COVID-19 Drugs"
    % by Charles F. Manski and Aleksey Tetenov
    % June 2020
    % We thank Valentyn Litvin for helpful comments regarding the code.
    %
    % Compute Dunnett's test two-sided critical value
    % for trials with unequal sample sizes across arms.
    % Inputs:
    %   K>=2  : number of treatments (control + (K-1) treatments)
    %   N     : 1 x K vector of per-arm sample sizes
    %   alpha : two-sided significance level (e.g., 0.05)
    
    % Degrees of freedom for the computation of pooled standard deviation
    DF = sum(N)-K;
    % Correlation matrix assuming equal sample sizes
    rho = eye(K-1);
    rho = ( (1 + N(1)./N(2:K)')*(1 + N(1)./N(2:K)) ).^-0.5;
    % Replace all diagonal elements with ones
    rho = rho + diag(1 - diag(rho));
    % Define an anonymous function f equal to the probability
    % that the maximum absolute value of the t-statistic is below
    % t (evaluated using multivariate t-distribution cdf)
    % minus 1-alpha.
    f = @(t) mvtcdf(-t*ones(K-1,1),t*ones(K-1,1),rho,DF) - (1-alpha);
    % Search for the critical value on the interval [0.01 20]
    cv = fzero(f,[0.01 20]);
end

