% Code to accompany "Statistical Decision Properties of Imprecise 
% Trials Assessing COVID-19 Drugs"
% by Charles F. Manski and Aleksey Tetenov
% June 2020
% We thank Valentyn Litvin for helpful comments regarding the code.
%
% Computing near-optimality for 5-arm trials with 2:1:1:1:1 sample ratio.
% Step 2/3
% This script performs a finer grid search to estimate near-optimality 
% of the empirical success rule and of Dunnett's test rule in trials
% with 5 treatment arms. It is assumed that arms 2-5 have the same
% sample size.
% The results of a global grid search suggest that largest expected loss
% for the empirical success rule occurs when the parameters (average
% success rates of treatments) have the form
% [a b b b b], a > b
% and the largest expected loss for Dunnett's test rule occurs
% for parameters of the form
% [a b c c c], b > a, b > c
% The script constructs two grids of parameter values in increments of 0.01
% having these forms and searches over them.
% Expected loss is approximated by simulating 1,000,000 trial outcomes
% for each set of parameter values.
% Requires MATLAB Optimization Toolbox for computing the critical value
% of Dunnett's test.
%
% Runtime and memory use:
% N = [60 60 60 60 60], N = [100 50 50 50 50]
% ~6.5 hours, ~4GB (in command-line mode)
% N = [2400 2400 2400 2400 2400], N = [4000 2000 2000 2000]
% ~6.5 hours, ~7GB (in command-line mode)
% on a desktop with a 6-core 3.2GHz Intel Core i7-8700B CPU

clear all
maxNumCompThreads('automatic');
% Use a faster RNG
rng(0,'simdTwister');

% Log output
diary finegrid_4000.txt


% Number of treatment arms (arm 1 is standard care)
K = 5;
% Sample sizes in each arm
% This script is designed for trials with equal sample size in arms 2..K
N = [4000 2000 2000 2000 2000];
% Number of simulated trials
MC = 1000000;

% The grid of average outcome values for each treatment
gridpoints = [0:0.01:1];
ng = size(gridpoints,2);
% The total number of parameter combinations considered.
gridsize = ng*(ng-1)/2 + ng*(ng-1)*(2*ng-1)/6

% Create MC random draws of average outcomes from each distribution on the grid
% and for each sample size.
tic
bingrid = zeros(MC,K*ng);
for k=1:K
    % To reduce memory use for large sample sizes, outcomes are generated
    % by summing up samples of size Nks in blocks summing up to N(k).
    % The size of the "outcomes" matrix is limited to 1.6GB (200M elements)
    maxblock = ceil(200000000/MC);
    % number of blocks
    Nkb = ceil(N(k)/maxblock);
    for b = 1:Nkb
        % Number of outcomes in this block
        Nks = min(maxblock, N(k)-(b-1)*maxblock);

        outcomes = rand(MC,Nks);
        for j=1:ng
            bingrid(:,K*(j-1)+k) = bingrid(:,K*(j-1)+k) + sum(outcomes<gridpoints(j),2)./N(k);
        end
    end
end
toc

% Free up memory
clear outcomes

pgrid = zeros(gridsize,5); % matrix of parameter combinations
igrid = pgrid; % matrix of indices in the parameter grid
gp = 1;
% First grid: [a b b b b], a > b
for i1 = 2:ng
    p1 = gridpoints(i1);
    % Only search over states in with p1 > p2 = p3 = p4 = p5
    for i2 = 1:(i1-1)
        p2 = gridpoints(i2);
        pgrid(gp,:) = [p1 p2 p2 p2 p2];
        igrid(gp,:) = [i1 i2 i2 i2 i2];
        gp = gp+1;
    end
end
% Second grid: [a b c c c], b > a, b > c
for i1 = 1:ng-1
    p1 = gridpoints(i1);
    % Only search over states in with p1 > p2 = p3 = p4 = p5
    for i2 = (i1+1):ng
        p2 = gridpoints(i2);
        for i3 = 1:(i2-1)
            p3 = gridpoints(i3);
            pgrid(gp,:) = [p1 p2 p3 p3 p3];
            igrid(gp,:) = [i1 i2 i3 i3 i3];
            gp = gp+1;
        end
    end
end

% Two-sided Dunnett's test critical value
tic
Dunnett_CV = dunnett_unequal(K,N,0.05);
toc

% Vectors to store results
NO_ES = zeros(gridsize,1);
NO_D = NO_ES;

% Vector to keep track of last simulated parameters
OldP = NaN*N;

% Prepare constants for calculations of t-statistics
DF = (sum(N)-K);
tstatc = ((N(1).^-1 + N(2:K).^-1).^-.5);

phats = zeros( MC, K);
% Flag whether a completion time estimate has been reported
no_time_est = true;
tic
for i=1:gridsize
    P = pgrid(i,:);
    for k=1:K
        if (P(k)~=OldP(k))
            phats(:,k) = bingrid(:,K*(igrid(i,k)-1)+k);
        end
    end
    OldP = P;

    % In each trial (row) find the set of treatments which have
    % the highest empirical success. Converted to double type
    % to speed up operations later.
    bests = double( phats == max(phats,[],2) );    

    % Allocate treatment equally to all best-performing treatments in each
    % trial and then average across MC trials.
    P_ES = mean ( bests./sum(bests,2) );
    NO_ES(i) = (max(P)-P) * P_ES';
    
    % Compute pooled outcome standard deviation for Dunnett's test.
    sdhats = (sum( N.*phats.*(1-phats), 2 )./DF ).^0.5;
    
    % Compute t-stats for comparing treatments 2:K with treatment 1
    tstats = tstatc .* ( ( phats(:,2:K)-phats(:,1) )./sdhats );

    % Find the set of treatments with t-stats > Dunnett's test cv
    % and having the largest average outcomes in each trial
    best_significant = bests(:,2:K).*( tstats > Dunnett_CV );

    % Find the set of treatments selected based on Dunnett's test:
    % if t-statistics for treatments 2..K are below the critical value,
    % then treatment 1 is chosen. Otherwise, treatment is equally allocated
    % among arms with the largest average outcomes.
    % max(1,sum(...)) avoids dividing by zero.
    P_D = mean( best_significant./max(1,sum(best_significant,2)) );
    % when any_significant = 0 (then best_significant is all zeros).
    % Average treatment assignment across simulated trials.
    NO_D(i,1) = (max(P)-P) * [1-sum(P_D) P_D]';

    % After 300 sec. provide an estimate of the remaining runtime.
    time_elapsed = toc;
    if (time_elapsed>300) & no_time_est
        fprintf('Elapsed time:  %7.0f sec.\n',time_elapsed);
        fprintf('Est remaining: %7.0f sec.\n',time_elapsed*(gridsize-i)/i);
        no_time_est = false;
    end
end
toc

% Remove temporary variables
clear bingrid igrid i1 i2 i3 i4 i5 p1 p2 p3 p4 p5 tstatc i j k OldP phats P_ES P_D bests sdhats tstats best_significant

% Save the results for step 3
save('finegrid_4000_0_001_1_1M.mat');

% Output of the results
% Select states in which treatment 1 is optimal
t1opt = ( pgrid(:,1) >= max(pgrid(:,2:K),[],2) );
% Select states in which treatment 1 is sub-optimal
t1sub = ( pgrid(:,1) < max(pgrid(:,2:K),[],2) );

T = 20;
fprintf("Worst states for ES rule in which treatment 1 is optimal\n");
[Rs si_eo] = sort(NO_ES.*t1opt,'descend');
format short
pgrid(si_eo(1:T),:)
format long
[NO_ES(si_eo(1:T)) NO_D(si_eo(1:T))] 

fprintf("Worst states for ES rule in which treatment 1 is sub-optimal\n");
[Rs si_es] = sort(NO_ES.*t1sub,'descend');
format short
pgrid(si_es(1:T),:)
format long
[NO_ES(si_es(1:T)) NO_D(si_es(1:T))] 

fprintf("Worst states for Dunnett's test rule in which treatment 1 is optimal\n");
[Rs si_do] = sort(NO_D.*t1opt,'descend');
format short
pgrid(si_do(1:T),:)
format long
[NO_ES(si_do(1:T)) NO_D(si_do(1:T))] 

fprintf("Worst states for Dunnett's test rule in which treatment 1 is sub-optimal\n");
[Rs si_ds] = sort(NO_D.*t1sub,'descend');
format short
pgrid(si_ds(1:T),:)
format long
[NO_ES(si_ds(1:T))  NO_D(si_ds(1:T))] 

diary off
