function [NO_ES,P_ES,NO_D,P_D] = nearopt_multiarm_specific(N,P,MC,Dunnett_CV)
    % Code to accompany "Statistical Decision Properties of Imprecise 
    % Trials Assessing COVID-19 Drugs"
    % by Charles F. Manski and Aleksey Tetenov
    % June 2020
    % We thank Valentyn Litvin for helpful comments regarding the code.
    %
    % This function evaluates near-optimality for multi-arm trials
    % enrolling prescpecified numbers of patients into each arm
    % for specific average outcome values.
    % Near-optimality is evaluated for two decision criteria:
    %   1. The empirical success rule: 
    %      treatment with the highest average outcome in the trial
    %      is prescribe. In case of a tie, each treatment with the highest
    %      average outcome is prescribed with equal probability.
    %   2. Two-sided Dunnett's test with critical value Dunnett_cv:
    %      treatment 1 is prescribed if all t-statistics comparing
    %      treatments 2:K to treatment 1 are below the critical value
    %      or negative. Otherwise, treatment with the highest average
    %      outcome in the trial is selected among treatments with t-stats
    %      exceeding the critical value.
    % Inputs:
    %   N     : 1xK vector N = [n_1 ... n_K] of sample sizes for each trial arm.
    %   P     : 1x2 vector P = [p_1 ... p_K] of average success rates of
    %           all treatments.
    %   MC    : number of trials to be simulated to compute near-optimality
    % Dunnett_CV : critical value for Dunnett's test.
    % Outputs:
    %   NO_ES : near-optimality of the empirical success (ES) rule
    %   P_ES  : probabilities of treatment assignment in the worst case
    %           for the ES rule
    %   NO_D  : near-optimality of Dunnett's test rule
    %   P_D   : probabilities of treatment assignment in the worst case
    %           for Dunnett's test rule
    %
    % Runtime and memory use depend on sample size, number of arms,
    % and number of simulations:
    % on a desktop with a 6-core 3.2GHz Intel Core i7-8700B CPU
    % with K = 5, MC = 100000000
    % N =  [500 250 250 250 250]: ~10 min., ~600MB
    % N = [1000 500 500 500 500]: ~20 min., ~  1GB

    % Infer the number of treatments
    K = size(N,2);
    
    % The simulation is split into blocks of 1000000 to limit memory use
    maxblock = 100000;
    MCb = ceil(MC/maxblock);
    % Store sums of treatment choice probabilities in each block, 
    % to be averaged at the end
    P_ESs  = zeros(MCb,K);
    P_Ds   = zeros(MCb,K);

    % Prepare constants for calculations of t-statistics
    DF = (sum(N)-K);
    tstatc = ((N(1).^-1 + N(2:K).^-1).^-.5);

    for b = 1:MCb
        % Number of simulations in this block
        MCs = min(maxblock, MC-(b-1)*maxblock);
        
        % Randomly draw MCs trial outcomes in each arm
        phats = zeros( MCs, K);
        for k = 1:K
            phats(:,k) = binornd( N(k), P(k), MCs, 1)./N(k);
        end

        % In each trial (row) find the set of treatments which have
        % the highest empirical success.
        bests = double( phats == max(phats,[],2) );    

        % Allocate treatment equally to all best-performing treatments
        % in each trial and sum the probabilities of each treatment
        % being chosen by the empirical success rule in this block
        % of simulations
        P_ESs(b,:) = sum ( bests./sum(bests,2) );

        % Compute pooled outcome standard deviation for Dunnett's test.
        sdhats = (sum( N.*phats.*(1-phats), 2 )./DF ).^0.5;

        % Compute t-stats for comparing treatments 2:K with treatment 1
        tstats = tstatc .* ( ( phats(:,2:K)-phats(:,1) )./sdhats );

        % Find the set of treatments with t-stats > Dunnett's test cv
        % and having the largest average outcomes in each trial
        best_significant = bests(:,2:K).*( tstats > Dunnett_CV );

        % Find the set of treatments selected based on Dunnett's test:
        % if t-statistics for treatments 2..K are below the critical value,
        % then treatment 1 is chosen. Otherwise, treatment is equally allocated
        % among arms with the largest average outcomes.
        % max(1,sum(...)) avoids dividing by zero.
        P_Ds(b,2:K) = sum( best_significant./max(1,sum(best_significant,2)) );
        P_Ds(b,1) = MCs-sum(P_Ds(b,2:K));
    end
    
    % Compute average assignment probabilities across all simulations
    % and near optimality
    P_ES = sum(P_ESs,1)./MC;
    NO_ES = (max(P)-P) * P_ES';
    P_D  = sum(P_Ds,1)./MC;
    NO_D = (max(P)-P) * P_D';
end