function [NO_ES,E_ES,P_ES,NO_T,E_T,P_T] = nearopt_twoarm(N,RANGE,ng,alpha)
    % Code to accompany "Statistical Decision Properties of Imprecise 
    % Trials Assessing COVID-19 Drugs"
    % by Charles F. Manski and Aleksey Tetenov
    % June 2020
    % We thank Valentyn Litvin for helpful comments regarding the code.
    %
    % This function evaluates near-optimality for two-armed trials
    % enrolling prescpecified numbers of patients into each arm.
    % Near-optimality is evaluated for two decision criteria:
    %   1. The empirical success rule: 
    %      treatment 2 is prescribed if it has a higher average outcome
    %      in the trial.
    %   2. Two-sided t-test with level alpha:
    %      treatment 2 is prescribed if the two-sided t-test rejects
    %      equality of means and treatment 2 has a higher average outcome
    %      in the trial.
    % Inputs:
    %   N     : 1x2 vector N = [n_1 n_2] of sample sizes for each trial arm.
    %   RANGE : 1x2 vector [lb,ub] of lower and upper bounds for the grid
    %   ng    : the number of grid points for the parameters,
    %           the gridpoints are equally spaced on [lb,ub] from RANGE.
    %   alpha : significance level of the two-sided hypothesis test.
    % Outputs:
    %   NO_ES : near-optimality of the empirical success (ES) rule
    %   E_ES  : average outcome probabilities in the worst case
    %           for the ES rule
    %   P_ES  : probabilities of treatment assignment in the worst case
    %           for the ES rule
    %   NO_T  : near-optimality of the t-test rule
    %   E_T   : average outcome probabilities in the worst case 
    %           for the t-test rule
    %   P_T   : probabilities of treatment assignment in the worst case
    %           for the t-test rule
    %
    % Runtime and memory use depend on sample size:
    % on a desktop with a 6-core 3.2GHz Intel Core i7-8700B CPU
    % N =   [1000 1000], ng = 1000:   280 seconds, 0.8GB
    % N =   [4000 4000], ng = 1000:  9900 seconds, 1.0GB
    % N = [10000 10000], ng = 1000; 64000 seconds, 2.3GB

    
    % Vectors of possible average outcomes of treatments 1 and 2
    % in the trial. 
    % Outcomes of the first arm are used as a column vector,
    % outcomes of the second arm are used as a row vector.
    % Hence, operations like (phats1 < phats2) return
    % an (N(1)+1) x (N(2)+1) matrix.
    nn1 = [0:N(1)]';
    nn2 = [0:N(2)];
    phats1 = nn1./N(1); 
    phats2 = nn2./N(2);
    
    % Empirical success rule as a function of trial outcomes
    % D_ES(.,.) = 1 for trial outcomes leading to treatment 2 prescription.
    D_ES = 0.5*(phats1 < phats2) + 0.5*(phats1 <= phats2);

    % Two-sample t-test two-sided critical value.
    DF = sum(N)-2;
    t_cv = tinv(1-alpha/2,DF);

    % Pooled outcome standard deviation for the t-test for all possible
    % trial realizations. 1e-15 added to avoid division by zero for trial
    % outcomes with zero all zeros or ones.
    sdhats = ( ( 1e-15 + N(1)*phats1.*(1-phats1) + N(2)*phats2.*(1-phats2) )./DF ).^0.5;

    % Compute t-stats for comparing treatment 2 with treatment 1
    % for all possible trial realizations
    tstats = ((N(1)^-1 + N(2)^-1)^-0.5)*(phats2-phats1)./sdhats;

    % Two-sided t-test treatment rule
    % D_T(.,.) = 1 when treatment 2 is prescribed
    % double(.) converts logical type into number to speed up further
    % operations
    D_T = double(tstats > t_cv);

    % Remove unnecessary arrays from memory
    clear sdhats tstats
    
    % Create the search grid on RANGE =[lb ub] with ng equally spaced points.
    gridpoints = RANGE(1) + (RANGE(2)-RANGE(1))*[1/(2*ng):1/ng:1];

    % Pre-compute the PDFs of trial outcomes for the second arm
    % for all parameters on the gridpoint
    % (to avoid redundant calculations in the loop).
    P2 = zeros(N(2)+1,ng);
    for i2 = 1:ng
        P2(:,i2) = binopdf(nn2',N(2),gridpoints(i2));
    end
    
    % Track highest values found:
    NO_ES = 0;
    NO_T  = 0;
    % Flag whether a completion time estimate has been reported
    no_time_est = true;
    tstart = tic;
    for i1 = 1:ng
        p1 = gridpoints(i1);
        % PDF of trial outcomes in the first arm
        P1 = binopdf(nn1',N(1),p1);
        for i2 = 1:ng
            p2 = gridpoints(i2);
            
            % ES rule: probability of choosing treatment 2
            P_ES_temp = P1*D_ES*P2(:,i2);
            % ES rule: expected loss
            NO_ES_temp = max(0,p2-p1) - (p2-p1)*P_ES_temp;
            % If higher average loss is found, record the solution
            if (NO_ES_temp > NO_ES)
                E_ES = [p1 p2];
                P_ES = [1-P_ES_temp P_ES_temp];
                NO_ES = NO_ES_temp;
            end
            
            % t-test rule: probability of choosing treatment 2
            P_T_temp = P1*D_T*P2(:,i2);
            % t-test rule: expected loss
            NO_T_temp = max(0,p2-p1) - (p2-p1)*P_T_temp;
            % If higher average loss is found, record the solution
            if (NO_T_temp > NO_T)
                E_T = [p1 p2];
                P_T = [1-P_T_temp P_T_temp];
                NO_T = NO_T_temp;
            end
        end
        % After 60 sec. provide an estimate of remaining runtime.
        time_elapsed = toc(tstart);
        if (time_elapsed>60) & no_time_est
            fprintf('Elapsed time:  %7.0f sec.\n',time_elapsed);
            fprintf('Est remaining: %7.0f sec.\n',time_elapsed*(ng-i1)/i1);
            no_time_est = false;
        end
    end
end
