function [NO_ES,P_ES,NO_T,P_T] = nearopt_twoarm_specific(N,P,alpha)
    % Code to accompany "Statistical Decision Properties of Imprecise 
    % Trials Assessing COVID-19 Drugs"
    % by Charles F. Manski and Aleksey Tetenov
    % June 2020
    % We thank Valentyn Litvin for helpful comments regarding the code.
    %
    % This function evaluates near-optimality for two-arm trials
    % enrolling prescpecified numbers of patients into each arm
    % for specific average outcome values in P.
    % Near-optimality is evaluated for two decision criteria:
    %   1. The empirical success rule: 
    %      treatment 2 is prescribed if it has a higher average outcome
    %      in the trial.
    %   2. Two-sided t-test with level alpha:
    %      treatment 2 is prescribed if the two-sided t-test rejects
    %      equality of means and treatment 2 has a higher average outcome
    %      in the trial.
    % Inputs:
    %   N     : 1x2 vector N = [n_1 n_2] of sample sizes for each trial arm.
    %   P     : 1x2 vector P = [p_1 p_2] of average success rates of
    %           the two treatments.
    %   alpha : significance level of the two-sided hypothesis test.
    % Outputs:
    %   NO_ES : near-optimality of the empirical success (ES) rule
    %   P_ES  : probabilities of treatment assignment in the worst case
    %           for the ES rule
    %   NO_T  : near-optimality of the t-test rule
    %   P_T   : probabilities of treatment assignment in the worst case
    %           for the t-test rule

    % Vectors of possible average outcomes of treatments 1 and 2
    % in the trial. 
    % Outcomes of the first arm are used as a column vector,
    % outcomes of the second arm are used as a row vector.
    % Hence, operations like (phats1 < phats2) return
    % an (N(1)+1) x (N(2)+1) matrix.
    nn1 = [0:N(1)]';
    nn2 = [0:N(2)];
    phats1 = nn1./N(1); 
    phats2 = nn2./N(2);
    
    % Empirical success rule as a function of trial outcomes
    % D_ES(.,.) = 1 for trial outcomes leading to treatment 2 prescription.
    D_ES = 0.5*(phats1 < phats2) + 0.5*(phats1 <= phats2);

    % Two-sample t-test two-sided critical value.
    DF = sum(N)-2;
    t_cv = tinv(1-alpha/2,DF);

    % Pooled outcome standard deviation for the t-test for all possible
    % trial realizations. 1e-15 added to avoid division by zero for trial
    % outcomes with zero all zeros or ones.
    sdhats = ( ( 1e-15 + N(1)*phats1.*(1-phats1) + N(2)*phats2.*(1-phats2) )./DF ).^0.5;

    % Compute t-stats for comparing treatment 2 with treatment 1
    % for all possible trial realizations
    tstats = ((N(1)^-1 + N(2)^-1)^-0.5)*(phats2-phats1)./sdhats;

    % Two-sided t-test treatment rule
    % D_T(.,.) = 1 when treatment 2 is prescribed
    % double(.) converts logical type into number to speed up further
    % operations
    D_T = double(tstats > t_cv);
    
    % PDF of trial outcomes in the two arms
    P1 = binopdf(nn1',N(1),P(1));
    P2 = binopdf(nn2',N(2),P(2));
        
    % ES rule: probability of choosing treatment 2
    P_ES_temp = P1*D_ES*P2;
    P_ES = [1-P_ES_temp P_ES_temp];
    % ES rule: expected loss
    NO_ES = (max(P)-P) * P_ES';
            
    % t-test rule: probability of choosing treatment 2
    P_T_temp = P1*D_T*P2;
    P_T = [1-P_T_temp P_T_temp];
    % t-test rule: expected loss
    NO_T = (max(P)-P) * P_T';
end
