% Code to accompany "Statistical Decision Properties of Imprecise 
% Trials Assessing COVID-19 Drugs"
% by Charles F. Manski and Aleksey Tetenov
% June 2020
% We thank Valentyn Litvin for helpful comments regarding the code.
%
% Computing near-optimality for 5-arm trials with 2:1:1:1:1 sample ratio.
% Step 3/3
% Take 10 parameter combinations with the largest estimated expected loss
% for each decision rule found in step 2 (finer grid search) and
% re-compute expected loss for these parameters with higher precision,
% by simulating 100,000,000 trial outcomes.
% We do this to verify that our results are not affected by bias resulting
% from approximating the sampling distribution by simulation.
% The results appearing in Table 4 of the paper are based on this output.
%
% Runtime and memory use:
% N = [60 60 60 60 60], N = [100 50 50 50 50]
% ~0.5 hour, ~250MB (in command-line mode)
% N = [2400 2400 2400 2400 2400], N = [4000 2000 2000 2000]
% ~25 hours, ~2.5GB (in command-line mode)
% on a desktop with a 6-core 3.2GHz Intel Core i7-8700B CPU

clear all
maxNumCompThreads('automatic');
% Use a faster RNG
rng(0,'simdTwister');

format long
% Log output
diary verify_4000.txt

% Load the results of finer grid search in step 2
% Per-arm sample size vector N and 
% Dunnett's test critical value Dunnett_CV are loaded in this step.
load('finegrid_4000_0_001_1_1M.mat');

% Number of simulated trials
MC = 100000000;

tic
% Candidate worst-case states for the empirical success rule
[~, si_es] = sort(NO_ES,'descend');
for t=1:10
    P = pgrid(si_es(t),:);
    [NO_ES,~,~,~] = nearopt_multiarm_specific(N,P,MC,Dunnett_CV);
    NO_ES
    toc
end


% Candidate worst-case states for the Dunnett's test rule
[~, si_d] = sort(NO_D,'descend');
for t=1:10
    P = pgrid(si_d(t),:);
    [~,~,NO_D,~] = nearopt_multiarm_specific(N,P,MC,Dunnett_CV);
    NO_D
    toc
end

diary off
