function [NO_ES,E_ES,P_ES,NO_D,E_D,P_D] = nearopt_multiarm(N,RANGE,ng,MC,alpha)
    % Code to accompany "Statistical Decision Properties of Imprecise 
    % Trials Assessing COVID-19 Drugs"
    % by Charles F. Manski and Aleksey Tetenov
    % June 2020
    % We thank Valentyn Litvin for helpful comments regarding the code.
    %
    % This function evaluates near-optimality for multi-arm trials
    % enrolling prescpecified numbers of patients into each arm.
    % Sample sizes in arms 2:K need not be equal, as a consequence,
    % unlike the grid search functions used to compute Table 4,
    % this function cannot exploit symmetry between arms and searches
    % over a complete grid of ng^K possible values of average outcomes
    % in each arm.
    % Near-optimality is evaluated for two decision criteria:
    %   1. The empirical success rule: 
    %      treatment with the highest average outcome in the trial
    %      is prescribe. In case of a tie, each treatment with the highest
    %      average outcome is prescribed with equal probability.
    %   2. Two-sided Dunnett's test with critical value Dunnett_cv:
    %      treatment 1 is prescribed if all t-statistics comparing
    %      treatments 2:K to treatment 1 are below the critical value
    %      or negative. Otherwise, treatment with the highest average
    %      outcome in the trial is selected among treatments with t-stats
    %      exceeding the critical value.
    % Inputs:
    %   N     : 1xK vector N = [n_1 ... n_K] of sample sizes for each trial arm.
    %   RANGE : 1x2 vector [lb,ub] of lower and upper bounds for the grid
    %   ng    : the number of grid points for the parameters,
    %           the gridpoints are equally spaced on [lb,ub] from RANGE.
    %   MC    : number of trials to be simulated to compute near-optimality
    %   alpha : significance level for two-sided Dunnett's test.
    % Outputs:
    %   NO_ES : near-optimality of the empirical success (ES) rule
    %   E_ES  : average outcome probabilities in the worst case
    %           for the ES rule
    %   P_ES  : probabilities of treatment assignment in the worst case
    %           for the ES rule
    %   NO_D  : near-optimality of Dunnett's test rule
    %   E_D   : average outcome probabilities in the worst case 
    %           for Dunnett's test rule
    %   P_D   : probabilities of treatment assignment in the worst case
    %           for Dunnett's test rule
    %
    % Requires MATLAB Optimization Toolbox for computing the critical value
    % of Dunnett's test.
    %
    % Runtime and memory use depend on the parameters:
    % N = [200 200 200], ng = 100, MC = 100000
    % ~40min, ~600MB (in command-line mode)
	% N = [2000 2000 2000 2000], ng = 30, MC = 100000
	% ~50min, ~3.2GB (in command-line mode)
    % on a desktop with a 6-core 3.2GHz Intel Core i7-8700B CPU

    % Infer number of arms from vector of sample sizes N
    K = size(N,2);
    
    % Create the search grid on RANGE =[lb ub] with ng equally spaced points.
    gridpoints = RANGE(1) + (RANGE(2)-RANGE(1))*[1/(2*ng):1/ng:1];
    
    % The total number of parameter combinations considered.
    gridsize = ng^K;

    % Create MC random draws of average outcomes from each distribution on the grid
    % and for each sample size.
    tic
    bingrid = zeros(MC,K*ng);
    for k=1:K
        % To reduce memory use for large sample sizes, outcomes are generated
        % by summing up samples of size Nks in blocks summing up to N(k).
        % The size of the "outcomes" matrix is limited to 1.6GB (200M elements)
        maxblock = ceil(200000000/MC);
        % number of blocks
        Nkb = ceil(N(k)/maxblock);
        for b = 1:Nkb
            % Number of outcomes in this block
            Nks = min(maxblock, N(k)-(b-1)*maxblock);

            outcomes = rand(MC,Nks);
            for j=1:ng
                bingrid(:,K*(j-1)+k) = bingrid(:,K*(j-1)+k) + sum(outcomes<gridpoints(j),2)./N(k);
            end
        end
    end
    toc

    % Free up memory
    clear outcomes

    % Create the matrix of indices for the parameter grid
    igridc = cell(1, K);
    [igridc{:}] = ndgrid(1:ng);
    igrid = zeros(gridsize,K);
    for i=1:K
        v = cell2mat(igridc(i));
        igrid(:,i) = v(:);
    end
    clear igridc v
    % Create the parameter grid
    pgrid = gridpoints(igrid);

    % Two-sided Dunnett's test critical value
    tic
    Dunnett_CV = dunnett_unequal(K,N,alpha);
    toc
    
    % Track highest values found:
    NO_ES = 0;
    NO_D  = 0;

    % Vector to keep track of last simulated parameters
    OldP = NaN*N;

    % Prepare constants for calculations of t-statistics
    DF = (sum(N)-K);
    tstatc = ((N(1).^-1 + N(2:K).^-1).^-.5);

    phats = zeros( MC, K);
    % Flag whether a completion time estimate has been reported
    no_time_est = true;
    tstart = tic;
    for i=1:gridsize
        P = pgrid(i,:);
        for k=1:K
            if (P(k)~=OldP(k))
                phats(:,k) = bingrid(:,K*(igrid(i,k)-1)+k);
            end
        end
        OldP = P;

        % In each trial (row) find the set of treatments which have
        % the highest empirical success. Converted to double type
        % to speed up operations later.
        bests = double( phats == max(phats,[],2) );    

        % Allocate treatment equally to all best-performing treatments in each
        % trial and then average across MC trials.
        P_ES_temp = mean ( bests./sum(bests,2) );
        NO_ES_temp = (max(P)-P) * P_ES_temp';
        % If higher average loss is found, record the solution
        if (NO_ES_temp > NO_ES)
            NO_ES = NO_ES_temp;
            E_ES = P;
            P_ES = P_ES_temp;
        end

            
        % Compute pooled outcome standard deviation for Dunnett's test.
        sdhats = (sum( N.*phats.*(1-phats), 2 )./DF ).^0.5;

        % Compute t-stats for comparing treatments 2:K with treatment 1
        tstats = tstatc .* ( ( phats(:,2:K)-phats(:,1) )./sdhats );

        % Find the set of treatments with t-stats > Dunnett's test cv
        % and having the largest average outcomes in each trial
        best_significant = bests(:,2:K).*( tstats > Dunnett_CV );

        % Find the set of treatments selected based on Dunnett's test:
        % if t-statistics for treatments 2..K are below the critical value,
        % then treatment 1 is chosen. Otherwise, treatment is equally allocated
        % among arms with the largest average outcomes.
        % max(1,sum(...)) avoids dividing by zero.
        P_D_temp = mean( best_significant./max(1,sum(best_significant,2)) );
        P_D_temp = [1-sum(P_D_temp) P_D_temp];
        % when any_significant = 0 (then best_significant is all zeros).
        % Average treatment assignment across simulated trials.
        NO_D_temp = (max(P)-P) * P_D_temp';
        % If higher average loss is found, record the solution
        if (NO_D_temp > NO_D)
            NO_D = NO_D_temp;
            E_D = P;
            P_D = P_D_temp;
        end

        % After 300 sec. provide an estimate of the remaining runtime.
        time_elapsed = toc(tstart);
        if (time_elapsed>300) & no_time_est
            fprintf('Elapsed time:  %7.0f sec.\n',time_elapsed);
            fprintf('Est remaining: %7.0f sec.\n',time_elapsed*(gridsize-i)/i);
            no_time_est = false;
        end
    end
    toc

end